@@ -22,7 +22,6 @@
 #define NULL_MARKER 0
 #define NOT_NULL_MARKER 255
 
-
 /* #if GRIB_PTHREADS */
 #if 0
 static pthread_once_t once  = PTHREAD_ONCE_INIT;
@@ -57,7 +56,6 @@ static void init()
 }
 #endif
 
-
 static const char* mars_keys =
     "mars.date,mars.time,mars.expver,mars.stream,mars.class,mars.type,"
     "mars.step,mars.param,mars.levtype,mars.levelist,mars.number,mars.iteration,"
@@ -81,7 +79,6 @@ static char* get_key(char** keys, int* type)
     while (*p == ' ')
         p++;
 
-
     while (*p != 0 && *p != ':' && *p != ',')
         p++;
     if (*p == ':') {
@@ -249,7 +246,7 @@ static grib_index_key* grib_index_new_key(grib_context* c, grib_index_key* keys,
     next = (grib_index_key*)grib_context_malloc_clear(c, sizeof(grib_index_key));
     if (!next) {
         grib_context_log(c, GRIB_LOG_ERROR,
-                         "unable to allocate %d bytes",
+                         "unable to allocate %ld bytes",
                          sizeof(grib_index_key));
         *err = GRIB_OUT_OF_MEMORY;
         return NULL;
@@ -257,7 +254,7 @@ static grib_index_key* grib_index_new_key(grib_context* c, grib_index_key* keys,
     values = (grib_string_list*)grib_context_malloc_clear(c, sizeof(grib_string_list));
     if (!values) {
         grib_context_log(c, GRIB_LOG_ERROR,
-                         "unable to allocate %d bytes",
+                         "unable to allocate %ld bytes",
                          sizeof(grib_string_list));
         *err = GRIB_OUT_OF_MEMORY;
         return NULL;
@@ -997,8 +994,8 @@ grib_index* grib_index_read(grib_context* c, const char* filename, int* err)
 int grib_index_search_same(grib_index* index, grib_handle* h)
 {
     int err        = 0;
-    char buf[1024] = {0,};
-    size_t buflen = 1024;
+    char buf[STRING_VALUE_LEN] = {0,};
+    size_t buflen = STRING_VALUE_LEN;
     grib_index_key* keys;
     long lval   = 0;
     double dval = 0.0;
@@ -1016,7 +1013,7 @@ int grib_index_search_same(grib_index* index, grib_handle* h)
             if (err)
                 keys->type = GRIB_TYPE_STRING;
         }
-        buflen = 1024;
+        buflen = STRING_VALUE_LEN;
         switch (keys->type) {
             case GRIB_TYPE_STRING:
                 err = grib_get_string(h, keys->name, buf, &buflen);
@@ -1404,7 +1401,6 @@ int grib_index_add_file(grib_index* index, const char* filename)
         if (err) return err;
         field->length=length;
 
-
         if (field_tree->field) {
             grib_field* pfield=field_tree->field;
             while (pfield->next) pfield=pfield->next;
